@@ -403,38 +403,40 @@ class EngineUp(EngineState):
         :type new_data: HostedEngineData
         :type logger: logging.Logger
         """
+
+        local_status = new_data.stats.local["engine-health"]
+        if local_status["vm"] == engine.VMState.DOWN:
+            logger.info("Engine vm is running on another host")
+            return EngineDown(new_data)
+
+        if local_status["vm"] == engine.VMState.DOWN_UNEXPECTED:
+            logger.info("Engine vm was unexpectedly shut down")
+            return EngineUnexpectedlyDown(new_data)
+
         if new_data.best_engine_status["vm"] != engine.VMState.UP:
-            local_vm_state = new_data.stats.local["engine-health"]["vm"]
-            if local_vm_state == engine.VMState.DOWN:
-                logger.info("Engine vm is running on another host")
-                return EngineDown(new_data)
-
-            if local_vm_state == engine.VMState.DOWN_UNEXPECTED:
-                logger.info("Engine vm was unexpectedly shut down")
-                return EngineUnexpectedlyDown(new_data)
-
             logger.info("Engine vm may be running on another host")
             return EngineMaybeAway(new_data), fsm.NOWAIT
 
-        elif new_data.best_engine_host_id != new_data.host_id:
-            logger.info("Engine vm unexpectedly running on host %d",
-                        new_data.best_engine_host_id)
-            return EngineDown(new_data)
-        elif new_data.best_engine_status["detail"] ==\
-                vmstatus.MIGRATION_SOURCE:
+        if local_status["detail"] == vmstatus.MIGRATION_SOURCE:
             logger.info("Engine VM found migrating away")
             return EngineMigratingAway(new_data)
-        elif (new_data.best_score_host and
-              new_data.best_score_host["host-id"] != new_data.host_id and
-              new_data.best_score_host["score"] >= self.score(logger) +
-              self.MIGRATION_THRESHOLD_SCORE):
+
+        if new_data.best_engine_host_id != new_data.host_id:
+            logger.info("Engine vm unexpectedly running on host %d",
+                        new_data.best_engine_host_id)
+            return EngineStop(new_data)
+
+        if (new_data.best_score_host and
+                new_data.best_score_host["host-id"] != new_data.host_id and
+                new_data.best_score_host["score"] >= self.score(logger) +
+                self.MIGRATION_THRESHOLD_SCORE):
             logger.error("Host %s (id %d) score is significantly better"
                          " than local score, shutting down VM on this host",
                          new_data.best_score_host['hostname'],
                          new_data.best_score_host["host-id"])
             return EngineStop(new_data)
-        elif (new_data.best_engine_status["vm"] == engine.VMState.UP and
-              new_data.best_engine_status["health"] == engine.Health.BAD):
+
+        if local_status["health"] == engine.Health.BAD:
             return EngineUpBadHealth(new_data)
         else:
             logger.info("Engine vm running on localhost",
@@ -568,11 +570,16 @@ class EngineStop(EngineState):
         :type new_data: HostedEngineData
         :type logger: logging.Logger
         """
-        if (new_data.best_engine_status["vm"] != engine.VMState.UP or
-                new_data.best_engine_host_id != new_data.host_id):
+        local_state = new_data.stats.local['engine-health']
+        if local_state['vm'] != engine.VMState.UP:
             logger.info("Engine vm not running on local host")
             return EngineDown(new_data)
-        elif new_data.timeout_start_time is None:
+
+        if local_state['detail'] == vmstatus.PAUSED:
+            logger.info("Engine VM is paused, forcefully stopping.")
+            return EngineForceStop(new_data), fsm.NOWAIT
+
+        if new_data.timeout_start_time is None:
             if fsm.actions.STOP_VM():
                 return EngineStop(new_data), fsm.WAIT
             else:
